from bs4 import BeautifulSoup
import os
import config
import re
import numpy as np

from model import Mention, Entity


def from_isnotes(path):
    
    split_path = path.split(r"/")
    root, file_name = "/".join(split_path[:-1]), split_path[-1]
    
    mentions = []
    
    if file_name.endswith(".mmax"):
        
        print(file_name)
        
        file_root = file_name[:-5]
        
        basedata = BeautifulSoup(open(root+"/Basedata/"+file_root+"_words.xml", "r").read(), features="lxml")
        entities = BeautifulSoup(open(root+"/markables/"+file_root+"_entity_level.xml", "r").read(), features="lxml")
        coref = BeautifulSoup(open(root+"/markables/"+file_root+"_coref_level.xml", "r").read(), features="lxml")
        
        for entity in entities.contents[2].contents[0].contents[0].find_all():
            
            span = entity["span"]
            tokens = []
            mention_id = entity["id"]
            
            # comparative: either "outsidetext" or antecedent id
            comp_from = None
            if entity["information_status"] == "mediated":
                try:
                    if entity["mediated_type"] == "comparative":
                        comp_from = entity["comparative_type"]
                        if comp_from == "withintext":
                            comp_from = entity["comp_from"]
                except:
                    pass
            
            try:
                coref_set = coref.find("markable", {"span": span})["coref_set"]
            except:
                coref_set = None
            
            if ".." in span:
                
                span_ids = [int(re.sub("word_", "", word)) for word in span.split("..")]
                for i in range(span_ids[0],span_ids[1]+1):
                    token = basedata.find("word", {"id": "word_"+str(i)}).contents[0]
                    tokens.append(token)
                
                mentions.append(Mention( tokens, [span_ids[0],span_ids[1]], mention_id, coref_set, comp_from))
                
            else:
                
                token = basedata.find("word", {"id": span}).contents[0]
                tokens.append(token)
                
                word_id = int(span.split("_")[-1])
                mentions.append(Mention(tokens, [word_id, word_id], mention_id, coref_set, comp_from))
    
    return mentions

def from_conll(path):
    #pass
    conll = open(path, 'r')
    
    mentions = list()
    
    text = list()
    poss = list()
    
    word_id = 0
    mention_id = 0
    coref_id = 0
    
    comp_count = 0
    ment_count = 0
    
    for line in conll:
        if line.startswith("#begin document"):
            stack = list()
            coref_stack = dict()
        elif line.startswith("#end document"):
            if comp_count > 0:
                for m in mentions:
                    print(m)
            return (ment_count, comp_count, mentions)
        elif line == "\n":
            stack = list()
        else:
            naked_line = line.strip()
            split_line = naked_line.split()
            # Important indices:
            # 1:doc_id, 2:token_id,     3:token,
            # 4:POS,    5:syntax_tree, -1:Coref
        
            coref = split_line[-1].split("|")
            
            coref_stack = {**coref_stack, **{int(c): word_id for c in re.findall("\(([0-9]+)", split_line[-1])}}
            
            coref_spans = dict()
            for c in re.findall("([0-9]+)\)", split_line[-1]):
                if int(c) not in coref_stack:
                    raise Exception
                coref_spans[(coref_stack[int(c)], word_id)] = int(c)
            
            syntax = split_line[5].split("*")
            
            opening = [str(f) for f in re.findall("\(([A-Z]+)",syntax[0])]
            for paren in opening:
                stack.append((word_id, paren, split_line[3], split_line[4]))
            
            
            text.append(split_line[3])
            poss.append(split_line[4])
            
            closing = [str(f) for f in re.finditer("\)",syntax[1])]
            
            
            for paren in closing:
                start = stack[-1]
                if start[1] == "NP" or start[1] == "NML" or start[1] == "NX" or start[1] == "NAC":
                    ment_count += 1
                    
                    rules = list()
                    
                    rules.append(start[3]=="JJR")
                    rules.append(text[start[0]].lower() == "more" 
                                 and len(poss) > start[0]+1 
                                 and poss[start[0]+1] == "JJ")
                    rules.append(text[start[0]].lower() in config.COMPARATIVES)
                    rules.append(text[start[0]].lower() == "others")
                    
                    if any(rules) and not "than" in text[start[0]:word_id+1]:
                        comp_count += 1
                        comp_from = "undefined"
                    else:
                        comp_from = None
                    
                    if (start[0],word_id) in coref_spans:
                        coref_set = coref_spans[(start[0],word_id)]
                    else:
                        coref_set = None
                    
                    mentions.append(Mention(text[start[0]:word_id+1], [start[0], word_id], mention_id, coref_set, comp_from))
                    
                stack = stack[:-1]
                
                
                
                
                mention_id += 1
            
            
            word_id += 1
            
    return (ment_count, comp_count, mentions)
                
    
if __name__ == "__main__":
    # ISnotes
    #root = config.DATA_PATH
    #files = os.listdir(root)
    
    #for f in files:
        #get_mentions(root+f)
    
    # conll/OntoNotes
    root = config.CONLL_PATH
    
    files = []
    for x in os.walk(root):
        for y in x[2]:
            print(y)
            if y.endswith(".v4_gold_conll"):
                files.append(x[0]+"/"+y)
    
    mcount = 0
    ccount = 0
    dist = np.zeros(26)
    for f in files:
        plus_m, plus_c, mentions = from_conll(f)
        if plus_c > 0:
            dist[plus_c] += 1
            mcount += plus_m
            ccount += plus_c
    print(mcount, ccount)
    print(dist[1:])
    
