"""
Fetches all documents including comparative constructions.
"""

import os
import sys
import re

PATH = "/resources/corpora/multilingual/ontonotes-5.0-conll-2012/conll-2012/v4/data/train/data/english/"
TARGET_PATH = "/proj/zimmermann/comparatives/corpus/train/"
COMPARATIVES = ["other", "similar", "comparable", "different", "additional", "extra"]


def fetch(PATH, TARGET_PATH, qual="gold"):
    jjr_count = 0
    rbr_count = 0
    jj_count = 0
    others_count = 0
    for root, dirs, files in os.walk(PATH):
        for f in files:
            if f.endswith(".v4_{}_conll".format(qual)):
                with open(os.path.join(root, f), "r") as document:
                    current_word_id = 0
                    contains_comparative = 0
                    current_doc_text = ""
                    current_doc_id = 0
                    
                    last_pos = ""
                    last_word = ""
                    stack = []
                    np_with_jj = False
                    in_comp = False
                    buff="" #buffer saves annotation after * for next line
                    
                    for line in document:
                        if line.strip():
                            
                            if line.startswith("#"):
                                continue
                            
                            cols = line.strip().split()
                            
                            if int(cols[1]) == current_doc_id + 1:
                                
                                if contains_comparative:
                                    target = "{}{}_{}_{}".format(TARGET_PATH,str(contains_comparative),f,current_doc_id)
                                    open(target, "w").write(current_doc_text)
                                    #print("Writing to "+target+".")
                                    contains_comparative = 0
                                    
                                current_doc_text = ""
                                current_doc_id += 1
                                
                            current_doc_text += line
                            
                            synt = cols[5]
                        
                            new_tags = re.findall("[A-Z]*", synt)
                            open_brackets = synt.count("(")
                            closed_brackets = synt.count(")")
                            
                            stack+=new_tags
                            
                            if "NP" in new_tags:
                                np_start_word_id = current_word_id
                                np_with_jj = []
                                    
                            if cols[4] == "JJ" and "NP" in stack and cols[3] in COMPARATIVES:
                                np_with_jj.append(cols[3])
                                
                            
                            if cols[4] == "JJR" and "NP" in stack:
                                contains_comparative += 1
                                jjr_count += 1
                            elif cols[4] == "JJ" and last_pos == "RBR" and "NP" in stack:
                                contains_comparative += 1
                                rbr_count += 1
                            elif (cols[4] == "NN" or cols[4] == "NNS") and np_with_jj:
                                contains_comparative += 1
                                jj_count += 1
                            elif cols[3].lower() == "others" and "NP" in stack:
                                contains_comparative += 1
                                others_count += 1
                                
                            last_pos = cols[4]
                            last_word = cols[3]
                            
                            if closed_brackets > 0:
                                del stack[-1*closed_brackets:]
                            
                            current_word_id += 1
                            
                        else: 
                            current_doc_text += line
                    
                    #last document
                    if contains_comparative:
                        target = "{}{}_{}_{}".format(TARGET_PATH,str(contains_comparative),f,current_doc_id)
                        open(target, "w").write(current_doc_text)
                        
    print(jjr_count, rbr_count, jj_count, others_count)

if __name__ == "__main__":
    PATH = "/resources/corpora/multilingual/ontonotes-5.0-conll-2012/conll-2012/v4/data/train/data/english/"
    TARGET_PATH = "/proj/zimmermann/comparatives/corpus/train/"
    fetch(PATH, TARGET_PATH)
    PATH = "/resources/corpora/multilingual/ontonotes-5.0-conll-2012/conll-2012/v4/data/test/data/english/"
    TARGET_PATH = "/proj/zimmermann/comparatives/corpus/test/"
    fetch(PATH, TARGET_PATH, "auto")
    PATH = "/resources/corpora/multilingual/ontonotes-5.0-conll-2012/conll-2012/v4/data/development/data/english/"
    TARGET_PATH = "/proj/zimmermann/comparatives/corpus/dev/"
    fetch(PATH, TARGET_PATH)
